@@ -1,6 +1,6 @@
 package POOF::Collection;
 
-use 5.006;
+use 5.007;
 use strict;
 use warnings;
 
@@ -9,13 +9,12 @@ use base qw(POOF);
 use B::Deparse;
 use Attribute::Handlers;
 use Scalar::Util qw(blessed refaddr);
-use Data::Dumper;
 use Carp qw(croak confess);
 
 use POOF::Properties::Array;
 use POOF::DataType;
 
-our $VERSION = '1.0';
+our $VERSION = '1.2';
 our $TRACE = 1;
 our $RAISE_EXCEPTION = 'trap';
 
@@ -102,4 +101,42 @@ sub pParent
 }    
 
 1;
+__END__
+
+=head1 NAME
+
+POOF::Collection - Perl extension that provides stronger typing, encapsulation
+and inheritance but unlike POOF it has an array as the core structure.
+
+=head1 SYNOPSIS
+
+    package MyClass;
+    use base qw(POOF::Collection);
+  
+=head1 DESCRIPTION
+
+A collection is like a list but it allows for methods that manipulate its own
+contents.  It also enforces the type of its elements.  Most of the information
+on POOF applies to POOF::Collections except thing that relate to properties and
+groups since the collection class does not have properties or groups only collection
+elements.
+
+=head1 SEE ALSO
+
+POOF man page.
+
+=head1 AUTHOR
+
+Benny Millares <bmillares@cpan.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+Copyright (C) 2007 by Benny Millares
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself, either Perl version 5.8.8 or,
+at your option, any later version of Perl 5 you may have available.
+
+=cut
+
 
